﻿(function (factory) {
    // Module systems magic dance.

    if (typeof require === "function" && typeof exports === "object" && typeof module === "object") {
        // CommonJS or Node: hard-coded dependency on "knockout"
        factory(require("knockout"), exports);
    } else if (typeof define === "function" && define["amd"]) {
        // AMD anonymous module with hard-coded dependency on "knockout"
        define(["knockout", "exports"], factory);
    } else {
        // <script> tag: use the global `ko` object, attaching a `mapping` property
        factory(ko, ko.mapper = {});
    }
}(function (ko, exports) {
    var propertyContainer = {};
    //将属性名注册到容器中
    function addProperty(key, arrProperty) {
        var old = propertyContainer[key] || []; //获取原始数组
        for (var i in arrProperty) {
            var newPropertyName = arrProperty[i];//新的属性名
            var hasProperty = false;
            for (var j in old) { //查找是否存在
                if (old[j] === newPropertyName) {
                    hasProperty = true;
                    break;
                }
            }
            if (!hasProperty) old.push(newPropertyName);
        }
    }
    //判断对象是否为数组
    function isArray(obj) {
        return Object.prototype.toString.call(obj) === '[object Array]';
    }
    //判断对象是否为Json对象
    function isJson(obj) {
        var isjson = typeof (obj) == "object" && Object.prototype.toString.call(obj).toLowerCase() == "[object object]" && !obj.length;
        return isjson;
    }
    //更新值
    function updateValue(viewModel, key, value, parent) {
        //得到最终的属性
        var fn;
        if (!parent) {
            fn = viewModel[key];
        } else { //循环取出最终的属性名
            if (parent.length) {
                fn = viewModel[parent[0]];
            }
            for (var p in parent) {
                fn = fn[p];
            }
        } //若属性是方法，则已加入观察者列表，则调用该方法
        if (typeof fn === "function") {
            fn(value);
        }
    }
    //遍历data
    function each(viewModel, data, parent) {
        for (var key in data) {
            var value = data[key];
            //处理非数组对象类型，则递归
            if (typeof value === "object" && isJson(value)) {
                var nodeParent = parent || [];
                nodeParent.push(key);
                each(viewModel, value, nodeParent);
            } else {
                updateValue(viewModel, key, value, parent);
            }
        }
    }
    //将data合并到viewModel上,并忽略viewModel上未定义的观察属性
    exports.extend = function (viewModel, data) {
        if (viewModel && data) {
            each(viewModel, data);
        }
    };
    exports.extendJson = function (viewModel, jsonString /*, options, target*/) {
        var parsed = ko.utils.parseJson(jsonString);
        arguments[0] = viewModel;
        arguments[1] = parsed;
        return exports.fromJS.apply(this, arguments);
    };
    //批量注册观察属性
    exports.observable = function (viewModel, arrProterty,saveName) {
        if (viewModel && arrProterty && arrProterty.length) {
            for (var i in arrProterty) {
                viewModel[arrProterty[i]] = ko.observable();
            }
        }
        if (saveName) {
            propertyContainer[saveName] = arrProterty;
        }
    }
    //批量注册数组观察属性
    exports.observableArray = function (viewModel, arrProterty) {
        if (viewModel && arrProterty && arrProterty.length) {
            for (var i in arrProterty) {
                viewModel[arrProterty[i]] = ko.observableArray();
            }
        }
    }
    //获取ViewModel中指定结点下的所有属性值
    exports.getValue = function (viewModel, data) {
        var arrProterty = isArray(data) ? data : propertyContainer[data];
        if (!arrProterty || !arrProterty.length) {
            throw new Error("未能获取到需要读取的属性");
        }
        var json = {};//保存需要取出的属性值
        // 开始遍历  
        for (var p in arrProterty) {
            var mName = arrProterty[p];//方法名
            if (typeof viewModel[mName] == "function") {
                json[mName] = viewModel[mName]();
            }
        }
        return json;
    }
}));